/*jslint node:true, nomen:true, debug:true */
/*global should, describe, before, beforeEach, it, escape */

var request = require('superagent'), should = require('should'), express = require('express'), 
app = express(), _ = require('lodash'), async = require('async'),
activator = require('../lib'), r = require('superagent')(app), templates = __dirname+'/resources',
USERS = {
	"1": {id:"1",email:"me@you.com",password:"1234"}
},
users,
userModel = {
	find: function (login,cb) {
		var found = null;
		if (!login) {
			cb("nologin");
		} else if (users[login]) {
			cb(null,users[login]);
		} else {
			_.each(users,function (val) {
				if (val && val.email === login) {
					found = val;
					return(false);
				}
			});
			cb(null,[found]);
		}
	},
	save: function (id,model,cb) {
		_.extend(users[id],model);
		cb();
	}
}, 
reset = function () {
	users = _.cloneDeep(USERS);
},
url = "smtp://localhost:30111/gopickup.net/"+escape("GoPickup Test <test@gopickup.net>"),
createUser = function (req,res,next) {
	users["2"] = {id:"2",email:"you@me.com",password:"5678"};
	req.userid = "2";
};


before(function(){
  debugger;
});

describe('activator', function(){
	before(function(){
		app.post('/usersbad',activator.createActivate);
		app.post('/users',createUser,activator.createActivate);
		app.put('/users/:user/activate',activator.completeActivate);
		app.post('/passwordreset',activator.createPasswordReset);
		app.put('/passwordreset/:user',activator.completeActivate);
	});
	describe('not initialized', function(){
    it('activate should send 500', function(done){
			r.post('/users').type("json").send({name:"john"}).expect(500,done);
    });
    it('completeactivate should send 500', function(done){
			r.put('/users/1/activate').type("json").send({code:"12345"}).expect(500,done);
    });
    it('passwordreset should send 500', function(done){
			r.post('/passwordreset').type("json").send({name:"john"}).expect(500,done);
    });
    it('completepasswordreset should send 500', function(done){
			r.put('/passwordreset/1').type("json").send({password:"abcd",code:"12345"}).expect(500,done);
    });
	});
	describe('initialized', function(){
		before(function(){
		  activator.init({user:userModel,url:url,templates:templates});
		});
		beforeEach(reset);
	  describe('activate', function(){
			it('should send 500 for unknown user', function(done){
				r.post('/usersbad').expect(500,done);
			});
			it('should fail for known user but bad code', function(done){
				async.waterfall([
					function (cb) {r.post('/users').expect(201,cb);},
					function (res,cb) {r.put('/users/'+res[0].id+'/activate').type("json").send({code:"asasqsqsqs"}).expect(400,cb);}
				],done);
			});
			it('should succeed for known user', function(done){
				async.waterfall([
					function (cb) {r.post('/users').expect(201,cb);},
					/** CHECK THE EMAIL ***/
					function (res,cb) {r.put('/users/'+res[0].id+'/activate').type("json").send({code:res[0].code}).expect(200,cb);}
				],done);
			});
	  });
		describe('password reset', function(){
		  it('should send 404 for unknown email or ID', function(done){
				r.post("/passwordreset").type('json').send({user:"john@john.com"}).expect(404,done);
		  });
			it('should fail for known email but bad code', function(done){
				async.waterfall([
					function (cb) {r.post('/passwordreset').type('json').send({user:"you@me.com"}).expect(201,cb);},
					/** CHECK THE EMAIL ***/
					function (res,cb) {r.put('/passwordreset/'+res[0].id).type("json").send({code:"asasqsqsqs",password:"asasa"}).expect(400,cb);}
				],done);
			});
			it('should fail for known email with good code but missing new password', function(done){
				async.waterfall([
					function (cb) {r.post('/passwordreset').type('json').send({user:"you@me.com"}).expect(201,cb);},
					/** CHECK THE EMAIL ***/
					function (res,cb) {r.put('/passwordreset/'+res[0].id).type("json").send({code:res[0].code}).expect(400,cb);}
				],done);
			});
			it('should succeed for known ID', function(done){
				async.waterfall([
					function (cb) {r.post('/passwordreset').type('json').send({user:"1"}).expect(201,cb);},
					/** CHECK THE EMAIL ***/
					function (res,cb) {r.put('/passwordreset/'+res[0].id).type("json").send({code:res[0].code,password:"abcdefgh"}).expect(200,cb);}
				],done);
			});
			it('should succeed for known email', function(done){
				async.waterfall([
					function (cb) {r.post('/passwordreset').type('json').send({user:"me@you.com"}).expect(201,cb);},
					/** CHECK THE EMAIL ***/
					function (res,cb) {r.put('/passwordreset/'+res[0].id).type("json").send({code:res[0].code,password:"abcdefgh"}).expect(200,cb);}
				],done);
			});
		});
	});
});